var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import validator from 'validator';
/**
 * Process data from given field and prepare it for a form mutation.
 */
export function processField(name, value, required, options, textLabels, customValidationFunction) {
    var _a, _b;
    if (options === void 0) { options = {}; }
    var type = options.type, min = options.min, forceErrorMessage = options.forceErrorMessage;
    // If the value is an array, remove its empty values for safety.
    var valueIsArray = Array.isArray(value);
    var processedValue = valueIsArray
        ? value.filter(function (item) { return Number.isInteger(item) || item instanceof Object || item.length; })
        : value;
    var validation = null;
    var help = null;
    // VALIDATION - If any check will fail, raise error state and set help message.
    if (required && (!processedValue || (valueIsArray && processedValue.length === 0))) {
        // If the field is required and its value is empty, set an error. Otherwise
        // continue the validation.
        validation = 'error';
        help = textLabels.requiredField;
    }
    else if ((processedValue === null || processedValue === void 0 ? void 0 : processedValue.length) && processedValue.length > 0) {
        // Force error message if it is present and abandon further validation.
        if (forceErrorMessage) {
            return _a = {},
                _a[name] = {
                    value: processedValue,
                    validation: 'error',
                    required: required,
                    help: forceErrorMessage,
                },
                _a;
        }
        switch (type) {
            case 'email':
                if (!validator.isEmail(value)) {
                    validation = 'error';
                    help = textLabels.emailInvalid;
                }
                break;
            case 'url':
                if (!validator.isURL(value)) {
                    validation = 'error';
                    help = textLabels.urlInvalid;
                }
                break;
            case 'tel':
                if (!value.match(/^\s*(?:\+?(\d{1,3}))?([-. (]*(\d{3})[-. )]*)?((\d{3})[-. ]*(\d{2,4})(?:[-.x ]*(\d+))?)\s*$/g)) {
                    validation = 'error';
                    help = textLabels.phoneInvalid;
                }
                break;
            case 'postcode':
                if (!validator.isPostalCode(value, 'DE')) {
                    validation = 'error';
                    help = textLabels.postCodeInvalid;
                }
                break;
            case 'json':
                try {
                    JSON.parse(value);
                }
                catch (e) {
                    validation = 'error';
                    help = textLabels.jsonInvalid;
                }
                break;
            default:
                // Handle custom validation function.
                if (type && customValidationFunction) {
                    var error = customValidationFunction(value, type);
                    if (error) {
                        validation = 'error';
                        help = error;
                        // TODO: Multilanguage support for error message.
                    }
                }
                // Minimal length option support.
                if (min && processedValue.length < min) {
                    validation = 'error';
                    help = textLabels.minChars.replace(':length:', min.toString());
                }
                break;
        }
    }
    // If there is no error and value is not empty, indicate success state.
    if (validation !== 'error' && ((processedValue && processedValue.length > 0) ||
        (typeof value === 'object' && !Array.isArray(value)))) {
        validation = 'success';
    }
    return _b = {},
        _b[name] = {
            value: processedValue,
            validation: validation,
            required: required,
            help: help,
            type: type,
        },
        _b;
}
/**
 * Generate initial form state, where all values are undefined.
 */
export function initiateFormFields(fieldNames, required) {
    return fieldNames.reduce(function (acc, field) {
        var _a;
        return (__assign(__assign({}, acc), (_a = {}, _a[field] = {
            value: undefined,
            // undefined means that field is in the initial state.
            // It is being used in formIsInitiated variable inside <Form> component.
            // TODO: Consider adding additional property called `untouch`.
            validation: null,
            required: required && required.includes(field),
            help: null,
        }, _a)));
    }, {});
}
/**
 * Reset valdiation states of all fields in a form.
 */
export function updateFieldsRequirements(fieldsData, required) {
    var updatedFieldsData = {};
    Object.keys(fieldsData).forEach(function (key) {
        var _a = fieldsData[key], value = _a.value, help = _a.help;
        var isRequired = required && required.includes(key);
        updatedFieldsData[key] = {
            value: value,
            // If the field is not on required anymore, validation must be cleaned up.
            validation: (fieldsData[key].validation === 'error' && !isRequired) ? null : fieldsData[key].validation,
            help: help,
            required: isRequired,
        };
    });
    return updatedFieldsData;
}
/**
 * Parse option, which can be a string or an object, to return an object.
 */
export function parseOption(option) {
    var optionLabel = (typeof option === 'string') ? option : option.label;
    var optionValue = (typeof option === 'string') ? option : option.value;
    return { optionLabel: optionLabel, optionValue: optionValue };
}
/**
 * Update single checkbox value in a list of all checkboxes.
 */
export function checkboxHandler(checked, newValue, previousValues) {
    if (checked) {
        if (previousValues) {
            return __spreadArray(__spreadArray([], previousValues, true), [newValue], false);
        }
        return [newValue];
    }
    return previousValues.filter(function (item) { return item !== newValue; });
}
/**
 * Check whether whole form is filled correctly.
 */
export function formIsInvalid(fieldsData, fieldKeys) {
    if (fieldKeys === void 0) { fieldKeys = []; }
    // Check only fields of given keys, otherwise check whole form.
    var fieldsToCheck = fieldKeys.length ? fieldKeys : Object.keys(fieldsData);
    var requiredButEmpty = false;
    var hasAnyError = false;
    fieldsToCheck.forEach(function (key) {
        var _a = fieldsData[key], value = _a.value, validation = _a.validation, required = _a.required;
        if (required && (typeof value === 'undefined' ||
            (typeof value === 'string' && value === '') ||
            (Array.isArray(value) && value.length === 0) ||
            (typeof value === 'object' && !Array.isArray(value) && (value === null || !Object.keys(value).length)))) {
            requiredButEmpty = true;
        }
        if (validation === 'error') {
            hasAnyError = true;
        }
    });
    return requiredButEmpty || hasAnyError;
}
/**
 * Get values from all fields and organize them into API friendly format.
 */
export function getValues(fieldsData) {
    var values = {};
    Object.keys(fieldsData).forEach(function (key) {
        values[key] = fieldsData[key].value;
    });
    return values;
}
/**
 * Convert image url to image data format that is compatible with image upload
 * input.
 */
export function imageUrltoImageData(imageUrl) {
    if (imageUrl) {
        return {
            name: imageUrl.split('/').pop(),
            data: imageUrl,
            type: 'image',
        };
    }
    return null;
}
